@@ -225,6 +225,11 @@ is inserted before the cursor, the short name is expanded to its address."
        "newsgroup"
        nil))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Case
+;;;
+
 (defun mew-complete-case ()
   "Complete function for cases."
   (interactive)
@@ -238,6 +243,36 @@ is inserted before the cursor, the short name is expanded to its address."
      "case"
      nil)))
 
+(defun mew-circular-complete-case ()
+  (interactive)
+  (let (cases oldcase newcase insert-:)
+    (save-excursion
+      (if (search-backward "," nil t)
+	  (forward-char 1)
+	(beginning-of-line))
+      (if (looking-at mew-regex-case2)
+	  (progn
+	    (setq oldcase (mew-match-string 1))
+	    (delete-region (match-beginning 1) (match-end 1)))
+	(setq oldcase mew-case-default)
+	(setq insert-: t))
+      (if (setq cases (member oldcase mew-config-cases))
+	  (if (> (length cases) 1)
+	      (setq newcase (nth 1 cases))
+	    (setq newcase (car mew-config-cases)))
+	(setq newcase mew-case-default))
+      (if (string= newcase mew-case-default)
+	  (unless insert-: (delete-char 1))
+	(insert newcase)
+	(if insert-: (insert ":"))))
+    (if (or (= (point) (mew-minibuf-point-min))
+	    (save-excursion
+	      (forward-char -1)
+	      (looking-at "[:,]")))
+	(if (search-forward "," nil t)
+	    (forward-char -1)
+	  (goto-char (point-max))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Circular completion: C-cC-i
@@ -453,7 +488,7 @@ the name exists."
      ((stringp cmp) ;; (length all) > 1
       (insert cmp)
       (mew-complete-window-show all)
-      (if (and EXPAND-CHAR (funcall fhit cmp ALIST))
+      (if (and (mew-characterp EXPAND-CHAR) (funcall fhit cmp ALIST))
 	  (message
 	   (substitute-command-keys
 	    "To expand '%s', type '%c' then '\\<mew-draft-header-map>\\[mew-draft-header-comp]'.")
@@ -526,8 +561,7 @@ the name exists."
         (regex (concat "[^" (or sep mew-address-separator) "]")))
     (save-excursion
       (while (and (not (bobp))
-                  (string-match regex (mew-buffer-substring
-                                       (1- (point)) (point))))
+                  (string-match regex (mew-buffer-substring (1- (point)) (point))))
         (forward-char -1))
       (if (and here (not (re-search-forward (regexp-quote here) end t)))
           nil ;; "here" does not exist.
@@ -543,9 +577,7 @@ the name exists."
       (forward-char 1)
     (beginning-of-line))
   (prog1
-      (cons (mew-buffer-substring (save-excursion (beginning-of-line)
-						  (point))
-				  (point))
+      (cons (mew-buffer-substring (mew-minibuf-point-min) (point))
 	    (mew-buffer-substring (point) (point-max)))
     (delete-region (point) (point-max))))
 
