@@ -251,12 +251,14 @@ all marked messages before the current message."
      ((mew-folder-nntpp folder)
       (message "This command cannot be used in this folder"))
      ((mew-folder-remotep folder)
+      (message "Refiling and deleting...")
+      (sit-for 0)
       (mew-summary-exec-remote beg end))
      ((mew-virtual-p)
       (if (not (mew-thread-p))
 	  (message "Move to a physical folder first")
 	(let* ((bnm (mew-summary-folder-name 'ext))
-	       (fld (mew-virtual-to-folder bnm))
+	       (fld (mew-thread-to-folder bnm))
 	       (msg (mew-summary-message-number)))
 	  (if (not (and fld (get-buffer fld)))
 	      (message "No physical folder")
@@ -269,6 +271,8 @@ all marked messages before the current message."
 		     (substitute-command-keys
 		      "\\<mew-summary-mode-map>\\[mew-summary-exec]"))))))
      (t
+      (message "Refiling and deleting...")
+      (sit-for 0)
       (mew-summary-exec-local beg end)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -318,11 +322,9 @@ all marked messages before the current message."
 		  (when (= cnt 0)
 		    ;; opening...
 		    (mew-summary-lock t "Executing")
-		    (message "Refiling and deleting...")
 		    (mew-window-configure 'summary)
 		    (mew-current-set nil nil nil)
 		    (mew-decode-syntax-delete))
-		  ;; (sit-for 0) ;; xxx
 		  (setq cnt (1+ cnt))
 		  ;; refiling and deleting...
 		  (setq func-sanity (mew-markdb-func-sanity mark))
@@ -417,7 +419,9 @@ all marked messages before the current message."
 	(setq rmvs nil))
       (set-buffer-modified-p nil)
       (if modify
-	  (list refs rmvs kils)
+	  (progn
+	    (sit-for 0)
+	    (list refs rmvs kils))
 	(list refs rmvs)))))
 
 (defun mew-summary-exec-remote (beg end)
@@ -431,7 +435,6 @@ all marked messages before the current message."
 	     (refs (nth 0 info))
 	     (rmvs (nth 1 info))
 	     (kils (nth 2 info)))
-	(sit-for 0)
 	(if (or refs rmvs)
 	    (cond
 	     ((mew-folder-popp folder)
@@ -466,6 +469,8 @@ all marked messages before the current message."
        (t
 	(setq beg (point-min))
 	(setq end (point-max))))
+      (message "Refiling and deleting...")
+      (sit-for 0)
       (mew-summary-exec-offline-region beg end))))
 
 (defun mew-summary-exec-offline-region (beg end)
@@ -506,7 +511,10 @@ all marked messages before the current message."
 	    (when rmvs
 	      (insert "Messages to be deleted:\n")
 	      (insert "\t" (mew-join "," rmvs) "\n"))
-	    (save-buffer))
+	    (set-buffer-modified-p nil)
+	    (mew-frwlet
+	     mew-cs-dummy nil
+	     (write-region (point-min) (point-max) (buffer-file-name) nil 'no-msg)))
 	  (mew-remove-buffer buf)
 	  (if (mew-imap-trash-folder case)
 	      (let* ((mew-inherit-case case)
