@@ -22,6 +22,7 @@
 (defvar mew-header-mode-map   nil)
 (defvar mew-addrbook-mode-map nil)
 (defvar mew-input-map         nil)
+(defvar mew-input-folder-map  nil)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -312,7 +313,7 @@
 
 (unless mew-draft-mode-map
   (setq mew-draft-mode-map (make-sparse-keymap))
-  (set-keymap-parent mew-draft-mode-map mew-draft-body-map))
+  (mew-set-keymap-parent mew-draft-mode-map mew-draft-body-map))
 
 (defvar mew-draft-mode-toolbar-menu
   '("Attachment Commands"
@@ -480,18 +481,29 @@
 	(if (boundp 'minibuffer-local-map)
 	    (copy-keymap minibuffer-local-map)
 	  (make-sparse-keymap)))
-  (define-key mew-input-map " "    'mew-input-complete)
-  (define-key mew-input-map "\t"   'mew-input-complete)
-  (define-key mew-input-map "?"    'mew-input-complete)
-;;  (define-key mew-input-map "\177" 'backward-delete-char)
-;;  (define-key mew-input-map [delete] 'backward-delete-char)
-;;  (define-key mew-input-map "\C-h" 'mew-complete-backscroll)
-  (define-key mew-input-map "\r"   'mew-exit-minibuffer)
-  (define-key mew-input-map "\n"   'mew-exit-minibuffer)
-  (define-key mew-input-map ","    'mew-input-comma)
-  (define-key mew-input-map "\C-g" 'abort-recursive-edit)
-  (define-key mew-input-map "\M-p" 'previous-history-element)
-  (define-key mew-input-map "\M-n" 'next-history-element))
+  (define-key mew-input-map "\r"     'mew-exit-minibuffer)
+  (define-key mew-input-map "\n"     'mew-exit-minibuffer)
+  (define-key mew-input-map [return] 'mew-exit-minibuffer)
+  (define-key mew-input-map " "      'mew-input-complete)
+  (define-key mew-input-map "\t"     'mew-input-complete)
+  (define-key mew-input-map [tab]    'mew-input-complete)
+  (define-key mew-input-map "?"      'mew-input-complete)
+  (define-key mew-input-map ","      'mew-input-comma))
+
+(unless mew-input-folder-map
+  (setq mew-input-folder-map (copy-keymap mew-input-map))
+  (define-key mew-input-folder-map [t]      'mew-input-folder-self-insert)
+  (define-key mew-input-folder-map "\C-g"   'mew-abort-minibuffer2)
+  (define-key mew-input-folder-map "\r"     'mew-exit-minibuffer2)
+  (define-key mew-input-folder-map "\n"     'mew-exit-minibuffer2)
+  (define-key mew-input-folder-map [return] 'mew-exit-minibuffer2)
+  (define-key mew-input-folder-map "+"      'mew-input-folder-prefix)
+  (define-key mew-input-folder-map "%"      'mew-input-folder-prefix)
+  (define-key mew-input-folder-map "$"      'mew-input-folder-prefix)
+  (define-key mew-input-folder-map "-"      'mew-input-folder-prefix)
+  (define-key mew-input-folder-map "*"      'mew-input-folder-prefix)
+  (define-key mew-input-folder-map "\C-s"   'mew-input-folder-search-forward)
+  (define-key mew-input-folder-map "\C-r"   'mew-input-folder-search-backward))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
