@@ -63,6 +63,228 @@ at completions."
 ;;;
 
 (defvar mew-input-folder-hist nil)
+(defvar mew-input-folder-search-direction nil)
+(defvar mew-input-folder-search-key nil)
+(defvar mew-input-folder-search-match nil)
+(defvar mew-input-folder-search-original nil)
+
+(defun mew-input-folder-prefix ()
+  (interactive)
+  (if mew-input-folder-search-direction
+      (mew-input-folder-self-insert)
+    (let ((pos (- (point) (mew-minibuf-point-min))))
+      (cond
+       ((= pos 0)
+	(insert last-command-char))
+       ((and (= pos 1)
+	     (save-excursion
+	       (forward-char -1)
+	       (looking-at "[-+%$]"))) ;; excluding *
+	(forward-char -1)
+	(delete-char 1)
+	(insert last-command-char))
+       ((and (> pos 1)
+	     (save-excursion
+	       (forward-char -2)
+	       (looking-at "[:,][-+*%$]")))
+	(forward-char -1)
+	(delete-char 1)
+	(insert last-command-char))
+       (t
+	(insert last-command-char))))))
+
+(defun mew-input-folder-display (&optional msg)
+  (delete-region (mew-minibuf-point-min) (point-max))
+  (insert "(" (or mew-input-folder-search-match "") ") ")
+  (insert (or mew-input-folder-search-key ""))
+  (if msg (save-excursion (insert " [" msg "]"))))
+
+(defun mew-input-folder-self-insert ()
+  (interactive)
+  (let* ((str-vec (this-command-keys))
+	 last-str gfunc)
+    (cond
+     ((stringp str-vec)
+      (setq last-str str-vec)
+      (setq gfunc (lookup-key (current-global-map) str-vec)))
+     ((vectorp str-vec)
+      (setq gfunc (lookup-key (current-global-map) str-vec))
+      (unless gfunc
+	(setq str-vec (lookup-key function-key-map str-vec))
+	(if (vectorp str-vec)
+	    (setq gfunc (lookup-key (current-global-map) str-vec))))))
+    (if mew-input-folder-search-direction
+	(cond
+	 ((or (equal str-vec "\177") (equal str-vec [127]))
+	  (if (null mew-input-folder-search-key)
+	      (mew-input-folder-display "not allowed")
+	    (setq mew-input-folder-search-key
+		  (substring mew-input-folder-search-key 0 -1))
+	    (when (string= mew-input-folder-search-key "")
+	      (setq mew-input-folder-search-key nil)
+	      (setq mew-input-folder-search-match nil)
+	      (save-excursion
+		(set-buffer mew-input-folder-search-buf)
+		(cond
+		 ((eq mew-input-folder-search-direction 'forward)
+		  (goto-char (point-min)))
+		 ((eq mew-input-folder-search-direction 'backward)
+		  (goto-char (point-max))))))
+	    (mew-input-folder-display)))
+	 ((not (string-match "self-insert-command" (symbol-name gfunc)))
+	  (mew-input-folder-display "not allowed"))
+	 ((eq mew-input-folder-search-direction 'forward)
+	  (setq mew-input-folder-search-key
+		(concat mew-input-folder-search-key last-str))
+	  (mew-input-folder-search-forward-1))
+	 ((eq mew-input-folder-search-direction 'backward)
+	  (setq mew-input-folder-search-key
+		(concat mew-input-folder-search-key last-str))
+	  (mew-input-folder-search-backward-1)))
+      (cond
+       ((null gfunc)
+	())
+       ((string-match "self-insert-command" (symbol-name gfunc))
+	(insert last-command-char))
+       ((and (fboundp gfunc) (commandp gfunc))
+	(call-interactively gfunc))))))
+
+(defvar mew-input-folder-search-buf "*Mew folder search") ;; xxx
+
+(defun mew-input-folder-search-forward-1 (&optional again)
+  (let (no-match)
+    (save-excursion
+      (set-buffer mew-input-folder-search-buf)
+      (if again
+	  (forward-char (length mew-input-folder-search-key))
+	(beginning-of-line))
+      (if (search-forward mew-input-folder-search-key nil t)
+	  (progn
+	    (goto-char (match-beginning 0))
+	    (setq mew-input-folder-search-match
+		  (buffer-substring
+		   (save-excursion (beginning-of-line) (point))
+		   (save-excursion (end-of-line) (point)))))
+	(setq no-match t)
+	(if again
+	    (forward-char (- (length mew-input-folder-search-key)))
+	  (setq mew-input-folder-search-key
+		(substring mew-input-folder-search-key 0 -1)))))
+    (if no-match 
+	(mew-input-folder-display "no match")
+      (mew-input-folder-display))))
+
+(defun mew-input-folder-search-backward-1 (&optional again)
+  (let (no-match)
+    (save-excursion
+      (set-buffer mew-input-folder-search-buf)
+      (if again
+	  (forward-char (- (length mew-input-folder-search-key)))
+	(end-of-line))
+      (if (search-backward mew-input-folder-search-key nil t)
+	  (progn
+	    (goto-char (match-end 0))
+	    (setq mew-input-folder-search-match
+		  (buffer-substring
+		   (save-excursion (beginning-of-line) (point))
+		   (save-excursion (end-of-line) (point)))))
+	(setq no-match t)
+	(if again
+	    (forward-char (length mew-input-folder-search-key))
+	  (setq mew-input-folder-search-key
+		(substring mew-input-folder-search-key 0 -1)))))
+    (if no-match 
+	(mew-input-folder-display "no match")
+      (mew-input-folder-display))))
+
+(defun mew-input-folder-search-forward ()
+  (interactive)
+  (unless mew-input-folder-search-direction
+    (setq mew-input-folder-search-original
+	  (buffer-substring (mew-minibuf-point-min) (point-max))))
+  (cond
+   ((and mew-input-folder-search-direction (null mew-input-folder-search-key))
+    (mew-input-folder-display "no match"))
+   ((eq mew-input-folder-search-direction 'forward)
+    (mew-input-folder-search-forward-1 'again))
+   ((eq mew-input-folder-search-direction 'backward)
+    (setq mew-input-folder-search-direction 'forward)
+    (mew-input-folder-search-forward-1 'again))
+   (t
+    (setq mew-input-folder-search-direction 'forward)
+    (mew-input-folder-search-setup 'min))))
+
+(defun mew-input-folder-search-backward ()
+  (interactive)
+  (unless mew-input-folder-search-direction
+    (setq mew-input-folder-search-original
+	  (buffer-substring (mew-minibuf-point-min) (point-max))))
+  (cond
+   ((and mew-input-folder-search-direction (null mew-input-folder-search-key))
+    (mew-input-folder-display "no match"))
+   ((eq mew-input-folder-search-direction 'forward)
+    (setq mew-input-folder-search-direction 'backward)
+    (mew-input-folder-search-backward-1 'again))
+   ((eq mew-input-folder-search-direction 'backward)
+    (mew-input-folder-search-backward-1 'again))
+   (t
+    (setq mew-input-folder-search-direction 'backward)
+    (mew-input-folder-search-setup))))
+
+(defun mew-input-folder-search-setup (&optional min)
+  (let* ((case:fld (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
+	 (case (car (mew-folder-case case:fld)))
+	 (fld (mew-folder-folder case:fld))
+	 alist buf)
+    (mew-input-folder-display)
+    (if (or (null fld) (string= fld ""))
+	(setq fld (mew-proto case)))
+    (setq buf (get-buffer-create mew-input-folder-search-buf))
+    (cond
+     ((mew-folder-popp fld)
+      (setq alist (mew-pop-folder-alist)))
+     ((mew-folder-nntpp fld)
+      (setq alist (mew-nntp-folder-alist case)))
+     ((mew-folder-imapp fld)
+      (setq alist (mew-imap-folder-alist case)))
+     (t
+      (setq alist (mew-local-folder-alist))))
+    (save-excursion
+      (set-buffer buf)
+      (setq buffer-read-only t)
+      (mew-elet
+       (mew-erase-buffer)
+       (mapcar (lambda (x) (if (stringp (car x)) (insert (car x) "\n"))) alist))
+      (if min (goto-char (point-min))))))
+
+(defun mew-exit-minibuffer2 ()
+  (interactive)
+  (if mew-input-folder-search-direction
+      (progn
+	(delete-region (mew-minibuf-point-min) (point-max))
+	(cond
+	 (mew-input-folder-search-match
+	  (insert mew-input-folder-search-match))
+	 (mew-input-folder-search-original
+	  (insert mew-input-folder-search-original)))
+	(mew-input-folder-clean-up))
+    (exit-minibuffer)))
+
+(defun mew-abort-minibuffer2 ()
+  (interactive)
+  (if mew-input-folder-search-direction
+      (progn
+	(delete-region (mew-minibuf-point-min) (point-max))
+	(when mew-input-folder-search-original
+	  (insert mew-input-folder-search-original))
+	(mew-input-folder-clean-up))
+    (abort-recursive-edit)))
+
+(defun mew-input-folder-clean-up ()
+  (setq mew-input-folder-search-direction nil)
+  (setq mew-input-folder-search-key nil)
+  (setq mew-input-folder-search-match nil)
+  (setq mew-input-folder-search-original nil))
 
 (defun mew-input-folder (default &optional local-only)
   "Input a folder from the minibuffer.
@@ -70,15 +292,14 @@ DEFAULT must be a folder WITHOUT case.
 This function returns a folder."
   (let ((prefix (mew-folder-prefix default))
 	(mew-inherit-case (mew-sinfo-get-case))
-	 folder)
+	folder)
     (mew-input-clear)
+    (mew-input-folder-clean-up)
     (if local-only
 	(setq mew-input-complete-function 'mew-complete-local-folder)
       (setq mew-input-complete-function 'mew-complete-folder))
     (setq folder (read-from-minibuffer (format "Folder name (%s): " default)
-				       prefix
-				       mew-input-map
-				       nil
+				       prefix mew-input-folder-map nil
 				       'mew-input-folder-hist))
     (when (or (string= folder "") (member folder mew-folder-prefixes))
       (setq folder default))
@@ -98,15 +319,14 @@ This function returns a folder."
       (setq prompt "Folder name: ")
       (setq init (mew-join "," default))))
     (mew-input-clear)
+    (mew-input-folder-clean-up)
     (if proto
 	(if (mew-folder-imapp proto)
 	    (setq mew-input-complete-function 'mew-complete-imap-folder)
 	  (setq mew-input-complete-function 'mew-complete-local-folder))
       (setq mew-input-complete-function 'mew-complete-folder))
     (setq folders (read-from-minibuffer prompt
-					init
-					mew-input-map
-					nil
+					init mew-input-folder-map nil
 					'mew-input-folder-hist))
     (when (and (stringp default) (string= folders init))
       (setq folders default))
@@ -185,9 +405,7 @@ This function returns a folder."
 (defvar mew-input-sort-key-hist nil)
 
 (defun mew-sort-key-check ()
-  (let* ((field:mode (mew-buffer-substring 
-		      (save-excursion (beginning-of-line) (point))
-		      (point-max)))
+  (let* ((field:mode (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
 	 (mode (car (cdr (mew-split field:mode ?:))))
 	 err)
     (if mode
@@ -430,9 +648,7 @@ This function returns a folder."
 ;;;
 
 (defun mew-case-check ()
-  (let* ((case (mew-buffer-substring 
-		(save-excursion (beginning-of-line) (point))
-		(point-max)))
+  (let* ((case (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
          (lst (mew-split case ?,))
 	 err)
     (catch 'nomatch
