@@ -52,23 +52,37 @@
       (let ((substatus (format "%d/%d" dcnt dttl)))
 	(mew-net-status buf "Deleting" substatus)))))
 
+(defvar mew-net-status-percent-size 51200)
+
 (defun mew-net-status2 (buf rttl rcnt siz &optional zero)
   (when mew-use-net-status
     (when (<= rcnt rttl)
       (let ((msiz (string-to-int siz))
 	    (bsiz (if zero 0 (buffer-size)))
 	    percent substatus)
-	(if (= msiz 0) (setq msiz 1))
-	(if (< 10000 msiz)
-	    (setq percent (/ bsiz (/ msiz 100)))
-	  (setq percent (/ (* bsiz 100) msiz)))
-	(setq substatus (format "%d/%d:%02d%%" rcnt rttl percent))
-	(mew-net-status buf "Retrieving" substatus)))))
+	(if (and (numberp mew-net-status-percent-size)
+		 (< msiz mew-net-status-percent-size))
+	    (progn
+	      (setq substatus (format "%d/%d" rcnt rttl))
+	      (mew-net-status buf "Retrieving" substatus))
+	  (if (= msiz 0) (setq msiz 1))
+	  (if (< 10000 msiz)
+	      (setq percent (/ bsiz (/ msiz 100)))
+	    (setq percent (/ (* bsiz 100) msiz)))
+	  (setq substatus (format "%d/%d:%02d%%" rcnt rttl percent))
+	  (mew-net-status buf "Retrieving" substatus))))))
+
+(defvar mew-local-status-unit 10)
+(defvar mew-local-status-threshold 100)
 
 (defun mew-net-status3 (buf rttl rcnt)
   (when mew-use-net-status
     (if rttl
-	(mew-net-status buf "Scanning" (format "%d/%d" rcnt rttl))
+	(if (or (not (numberp mew-local-status-unit))
+		(not (numberp mew-local-status-threshold))
+		(< (- rttl rcnt) mew-local-status-threshold)
+		(= 0 (% rcnt mew-local-status-unit)))
+	    (mew-net-status buf "Scanning" (format "%d/%d" rcnt rttl)))
       (mew-net-status buf "Scanning" (format "%d" rcnt)))))
 
 (defun mew-summary-visible-buffer (buf)
@@ -202,11 +216,12 @@
 (defun mew-queue-enqueue (work qfld) ;; 1.wrk or draft/1
   (let (orig info iwrk)
     (if (not (string-match (concat mew-queue-work-suffix "$") work))
+	;; draft/1
 	(setq orig (mew-folder-new-message qfld))
+      ;; 1.wrk
       (setq orig (file-name-sans-extension work)) ;; 1
       (setq iwrk (concat orig mew-queue-info-suffix mew-queue-work-suffix))
-      ;; 1.mqi.wrk
-      (mew-delete-file iwrk)
+      (mew-delete-file iwrk) ;; 1.mqi.wrk
       (if (file-exists-p orig) (setq orig (mew-folder-new-message qfld))))
     (rename-file work orig 'override)
     (unless mew-use-nfs-hack
@@ -230,9 +245,11 @@
   (let* ((work (concat file mew-queue-work-suffix)) ;; 1.wrk
 	 (info (concat file mew-queue-info-suffix)) ;; 1.mqi
 	 (iwrk (concat info mew-queue-work-suffix)) ;; 1.mqi.wrk
-	 (i 0) data)
+	 (i 0) buf data)
     (when (and (file-readable-p file) (file-readable-p info))
       (rename-file file work 'override)
+      ;; If an old buffer exists by accident, we MUST remove the buffer.
+      (if (setq buf (get-file-buffer work)) (mew-remove-buffer buf))
       (mew-frwlet
        mew-cs-text-for-read mew-cs-dummy
        (set-buffer (find-file-noselect work)))
